from osv import osv, fields


# Service Order
class ServiceSystem(osv.osv):
    _name = "service.system"
    _description = "System"
    _inherit = ["mail.thread"]

    _columns = {
        'name': fields.char('Name', size=32, required=True,
                            track_visibility='always'),
        'component_ids': fields.one2many(
            'service.component', 'system_id', 'Components',
            track_visibility='always'),
        'partner_id': fields.many2one('res.partner', 'Partner',
                                      track_visibility='always'),
        'sector_id': fields.many2one('res.sector', 'Sector',
                                     track_visibility='always'),
        'service_order_ids': fields.one2many('service.order', 'system_id',
                                             'Service Orders',
                                             track_visibility='always'),
        'active': fields.boolean('Active',
                                 help="Uncheck this if you want to mark " +
                                 "the system as withdrawn",
                                 track_visibility='always'),
        'id_sistema': fields.integer("Id CM", track_visibility='always'),
        'installation_date': fields.date("Installation Date",
                                         track_visibility='always'),
        'write_date': fields.datetime('Last Update', readonly=True),
        'system_type_id': fields.many2one('service.system.type',
                                          'System type'),
        'child_system_type_ids': fields.many2many(
            'service.system.type', 'child_system_type_id', 'system_id',
            'system_type_id', 'System type',
            domain="[('parent_id','=', system_type_id)]"),
        'observations': fields.text(
            string='Observations'),
        'service_order_type_ids': fields.one2many('service.order.type.system',
                                                  'service_system_id',
                                                  'Service Order Type by System'),
        'cleaning_date': fields.date("Cleaning Date",
                                         track_visibility='always'),
        'equipment_gps_location': fields.text(
            string='Equipment GPS location',
            help="To get the Google Maps link, go to the Google Maps web, "
            "set and adress and then go to share -> copy link"
            )
    }
    _defaults = {
        'active': lambda *x: True,
    }

    def onchange_partner_id(self, cr, uid, ids, partner_id, sector_id):
        ret = {
            'value': {}
        }
        # pd = {
        #     'partner_id': partner_id,
        # }
        # system = self.browse(cr, uid, ids[0])
        # ret['value']['component_ids'] = [(1, x.id, pd) for x
        #                                  in system.component_ids]
        sector_obj = self.pool['res.sector']
        if sector_id:
            sector = sector_obj.browse(cr, uid, sector_id)
            if sector.partner_id.id != partner_id:
                ret['value']['sector_id'] = False
        if not partner_id:
            return ret

        ret['domain'] = {
            'sector_id': [('partner_id', '=', partner_id)]
        }
        return ret


class ServiceSystemType(osv.osv):
    _name = 'service.system.type'

    def name_get(self, cr, uid, ids, context=None):
        if context is None:
            context = {}
        reads = self.read(cr, uid, ids, ['name', 'parent_id'],
                          context=context)
        res = []
        for record in reads:
            name = record['name']
            if record['parent_id']:
                name = record['parent_id'][1]+' / '+name
            res.append((record['id'], name))
        return res

    def name_search(self, cr, uid, name, args=None,
                    operator='ilike', context=None, limit=100):
        if not args:
            args = []
        if not context:
            context = {}
        if name:
            # Be sure name_search is symetric to name_get
            name = name.split(' / ')[-1]
            ids = self.search(cr, uid, [('name', operator, name)] + args,
                              limit=limit, context=context)
        else:
            ids = self.search(cr, uid, args, limit=limit, context=context)
        return self.name_get(cr, uid, ids, context)

    def _name_get_fnc(self, cr, uid, ids, prop,
                      unknow_none, context=None):
        res = self.name_get(cr, uid, ids, context=context)
        return dict(res)

    _columns = {
        'name': fields.char('System Type', required=True),
        'parent_id': fields.many2one('service.system.type',
                                     'System type related'),
        'complete_name': fields.function(_name_get_fnc, type="char",
                                         string='Full Name'),
        'child_ids': fields.one2many('service.system.type', 'parent_id',
                                     string='Child Categories'),
        'parent_left': fields.integer('Left Parent', select=1),
        'parent_right': fields.integer('Right Parent', select=1),
        'module_ids': fields.many2many(
            'service.module', 'service_system_type_module_rel',
            'system_type_id', 'module_id', string='Modules'),
        'technique': fields.char('Technique'),
        'subtechnique': fields.char('SubTechnique'),
        'eng_technique': fields.char('Technique in English'),
    }
