# -*- encoding: utf-8 -*-
##############################################################################
#
#    OpenERP, Open Source Management Solution
#    Copyright (c) 2016 E-MIPS (http://www.e-mips.com.ar)
#    Copyright (c) 2016 Eynes (http://www.eynes.com.ar)
#    All Rights Reserved. See AUTHORS for details.
#
#    This program is free software: you can redistribute it and/or modify
#    it under the terms of the GNU Affero General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU Affero General Public License for more details.
#
#    You should have received a copy of the GNU Affero General Public License
#    along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
##############################################################################

import xlwt
from datetime import datetime, timedelta
from openerp.osv import orm
from openerp.report import report_sxw
from openerp.addons.report_xls.report_xls import report_xls
from openerp.addons.report_xls.utils import rowcol_to_cell, _render
from openerp.tools import DEFAULT_SERVER_DATE_FORMAT, DEFAULT_SERVER_DATETIME_FORMAT, DATETIME_FORMATS_MAP, float_compare
from openerp.tools.translate import translate, _
import logging
_logger = logging.getLogger(__name__)

_ir_translation_name = 'service.asignee.line.xls'

class assignee_report_xls_parser(report_sxw.rml_parse):

    def __init__(self, cr, uid, name, context):
        super(assignee_report_xls_parser, self).__init__(
            cr, uid, name, context=context)
        self.context = context
        self.localcontext.update({
            'datetime': datetime,
            '_': self._,
        })

    def _(self, src):
        lang = self.context.get('lang', 'en_US')
        return translate(self.cr, _ir_translation_name, 'report', lang, src) \
            or src

    def _get_days(self, date_start, date_end):

        if date_start > date_end:
            dt = date_start
            date_start = date_end
            date_end = dt

        date_start_dt = datetime.strptime(date_start, '%Y-%m-%d')
        date_end_dt = datetime.strptime(date_end, '%Y-%m-%d')

        delta = date_end_dt - date_start_dt
        delta = delta.days
        if delta > 30:
            delta = 30

        fields = []
        for i in range(0, delta):
            print i
            dd = date_start_dt+timedelta(days=i)
            fields.append(dd)

        return fields

    def set_context(self, objects, data, ids, report_type=None):
        objects = []

        assignee_obj = self.pool.get('service.order.assignee.line')
        self.context['from_date'] = data['from_date']
        self.context['to_date'] = data['to_date']

        days = self._get_days(self.context['from_date'], self.context['to_date'])

        wanted_list = ['name']
        for d in days:
           wanted_list.append(d.strftime(DEFAULT_SERVER_DATE_FORMAT))

        wanted_list += assignee_obj._report_xls_fields(self.cr, self.uid, self.context)
        template_changes = assignee_obj._report_xls_template(self.cr, self.uid, days, self.context)

        self.localcontext.update({
            'wanted_list': wanted_list,
            'template_changes': template_changes,
        })

        employee_obj = self.pool.get('hr.employee')
        line_obj = self.pool.get('service.order.assignee.line')

        employee_ids = employee_obj.search(self.cr, self.uid, [], context=self.context)
        employees = employee_obj.browse(self.cr, self.uid, employee_ids, context=self.context)

        for employee in employees:
            vals = {'name': employee.name}
            ldays = []

            for day in days:
                line_ids = line_obj.search(self.cr, self.uid,
                                [('employee_id', '=', employee.id),
                                 ('date', '<=', day.strftime(DEFAULT_SERVER_DATE_FORMAT)),
                                 ('date_end', '>=', day.strftime(DEFAULT_SERVER_DATE_FORMAT))])
                if line_ids:
                    lines = line_obj.browse(self.cr, self.uid, line_ids)
                    customers = [l.order_id.partner_id.name for l in lines]
                    customers = ' || '.join(customers)
                    ldays.append(customers)
                else:
                    ldays.append('')

            vals['days'] = ldays
            objects.append(vals)

        return super(assignee_report_xls_parser, self).set_context(
            objects, data, ids, report_type=report_type)

class assignee_calendar_xls(report_xls):

    def __init__(self, name, table, rml=False, parser=False, header=True,
                 store=False):
        super(assignee_calendar_xls, self).__init__(
            name, table, rml, parser, header, store)

        # Cell Styles
        _xs = self.xls_styles
        # header
        rh_cell_format = _xs['bold'] + _xs['fill'] + _xs['borders_all']
        self.rh_cell_style = xlwt.easyxf(rh_cell_format)
        self.rh_cell_style_center = xlwt.easyxf(rh_cell_format + _xs['center'])
        self.rh_cell_style_right = xlwt.easyxf(rh_cell_format + _xs['right'])
        # lines
        aml_cell_format = _xs['borders_all'] + _xs['wrap']
        self.aml_cell_style = xlwt.easyxf(aml_cell_format)
        self.aml_cell_style_center = xlwt.easyxf(
            aml_cell_format + _xs['center'])
        self.aml_cell_style_date = xlwt.easyxf(
            aml_cell_format + _xs['left'],
            num_format_str=report_xls.date_format)
        self.aml_cell_style_decimal = xlwt.easyxf(
            aml_cell_format + _xs['right'],
            num_format_str=report_xls.decimal_format)
        # totals
        rt_cell_format = _xs['bold'] + _xs['fill'] + _xs['borders_all']
        self.rt_cell_style = xlwt.easyxf(rt_cell_format)
        self.rt_cell_style_right = xlwt.easyxf(rt_cell_format + _xs['right'])
        self.rt_cell_style_decimal = xlwt.easyxf(
            rt_cell_format + _xs['right'],
            num_format_str=report_xls.decimal_format)

        # XLS Template
        self.col_specs_template = {
#            'name': {
#                'header': [1, 42, 'text', _render("_('Employee')")],
#                'lines': [1, 0, 'text', _render("line.employee_id.name or ''")],
#                'totals': [1, 0, 'text', None]},
#            'date': {
#                'header': [1, 13, 'text', _render("_('Date')")],
#                'lines': [1, 0, 'date',
#                    _render("datetime.strptime(line.date,'%Y-%m-%d %H:%M:%S')"),
#                          None, self.aml_cell_style_date],
#                'totals': [1, 0, 'text', None]},
#            'date_end': {
#                'header': [1, 13, 'text', _render("_('Date End')")],
#                'lines': [1, 0, 'date',
#                    _render("datetime.strptime(line.date_end,'%Y-%m-%d %H:%M:%S')"),
#                          None, self.aml_cell_style_date],
#                'totals': [1, 0, 'text', None]},
        }

    def generate_xls_report(self, _p, _xs, data, objects, wb):

        wanted_list = _p.wanted_list
        self.col_specs_template.update(_p.template_changes)
        _ = _p._

#        debit_pos = 'debit' in wanted_list and wanted_list.index('debit')
#        credit_pos = 'credit' in wanted_list and wanted_list.index('credit')
#        if not (credit_pos and debit_pos) and 'balance' in wanted_list:
#            raise orm.except_orm(
#                _('Customisation Error!'),
#                _("The 'Balance' field is a calculated XLS field requiring \
#                the presence of the 'Debit' and 'Credit' fields !"))

        # report_name = objects[0]._description or objects[0]._name
        report_name = _("Assignee Calendar")
        ws = wb.add_sheet(report_name[:31])
        ws.panes_frozen = True
        ws.remove_splits = True
        ws.portrait = 0  # Landscape
        ws.fit_width_to_pages = 1
        row_pos = 0

        # set print header/footer
        ws.header_str = self.xls_headers['standard']
        ws.footer_str = self.xls_footers['standard']

        # Title
        cell_style = xlwt.easyxf(_xs['xls_title'])
        c_specs = [
            ('report_name', 1, 0, 'text', report_name),
        ]
        row_data = self.xls_row_template(c_specs, ['report_name'])
        row_pos = self.xls_write_row(
            ws, row_pos, row_data, row_style=cell_style)
        row_pos += 1

        # Column headers
        c_specs = map(lambda x: self.render(
            x, self.col_specs_template, 'header', render_space={'_': _p._}),
            wanted_list)
        row_data = self.xls_row_template(c_specs, [x[0] for x in c_specs])
        row_pos = self.xls_write_row(
            ws, row_pos, row_data, row_style=self.rh_cell_style,
            set_column_size=True)
        ws.set_horz_split_pos(row_pos)

        # lines
        for line in objects:
            c_specs = map(
                lambda x: self.render(x, self.col_specs_template, 'lines'),
                wanted_list)
            row_data = self.xls_row_template(c_specs, [x[0] for x in c_specs])
            row_pos = self.xls_write_row(
                ws, row_pos, row_data, row_style=self.aml_cell_style)

        # No hacen falta los totales
#        c_specs = map(
#            lambda x: self.render(x, self.col_specs_template, 'totals'),
#            wanted_list)
#        row_data = self.xls_row_template(c_specs, [x[0] for x in c_specs])
#        row_pos = self.xls_write_row(
#            ws, row_pos, row_data, row_style=self.rt_cell_style_right)

assignee_calendar_xls('report.service.asignee.line.xls',
              'service.order.assignee.line',
              parser=assignee_report_xls_parser)
