# -*- encoding: utf-8 -*-
##############################################################################
#
#    OpenERP, Open Source Management Solution
#    Copyright (c) 2017 E-MIPS (http://www.e-mips.com.ar)
#    All Rights Reserved. See AUTHORS for details.
#
#    This program is free software: you can redistribute it and/or modify
#    it under the terms of the GNU Affero General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU Affero General Public License for more details.
#
#    You should have received a copy of the GNU Affero General Public License
#    along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
##############################################################################

from datetime import date, datetime
from dateutil import relativedelta
from openerp.tests import common
from openerp import netsvc
from openerp.osv.osv import except_osv
from functools import partial
from openerp.addons.service.tests.test_suitcase import CommonSuitcaseFunc
import unittest2


class TestSaleOrder(common.TransactionCase):
    def setUp(self):
        super(TestSaleOrder, self).setUp()
        cr, uid, context = self.cr, self.uid, {}

        self.registry('res.users').write(cr, uid, self.uid, {'lang': 'en_US'})

        # Datos a utilizar
        data_model = self.registry('ir.model.data')
        self.get_ref = partial(data_model.get_object_reference,
                               self.cr, self.uid)


        self.product_product = self.registry('product.product')
        self.sale_model = self.registry('sale.order')
        self.sale_line_model = self.registry('sale.order.line')
        self.service_order_model = self.registry('service.order')
        self.create_service_order_model = self.registry('sale.order.create.service')

        __, self.warehouse_id = self.get_ref('stock', 'warehouse0')
        __, self.product1_id = self.get_ref('product', 'product_product_30')
        __, self.supplier_asustek_id = self.get_ref('base', 'res_partner_1')
        __, self.location_components_id = self.get_ref('stock', 'stock_location_components')
        __, self.partner_id = self.get_ref('base', 'res_partner_7')
        __, self.sector_lab_id = self.get_ref('jenck_partner', 'sector_lab')
        __, self.employee1_id = self.get_ref('hr', 'employee_fp')
        __, self.employee_lab_id = self.get_ref('jenck_partner', 'employee_lab1')
        __, self.type_maintenance_id = self.get_ref('service', 'service_type_maintenance')
        __, self.type_installation_id = self.get_ref('service', 'service_type_installation')

    def create_sale_order(self, partner_id, product_id, qty):
        cr, uid, context = self.cr, self.uid, {}

        res = self.sale_model.onchange_partner_id(cr, uid, [],
                partner_id)

        so_vals = {
            'partner_id': partner_id,
            'sector_id': self.sector_lab_id,
            'employee_id': self.employee_lab_id,
            'technique_sector_id': self.sector_lab_id,
            'technique_employee_id': self.employee_lab_id,
            'type': 'place',
            #'pricelist_id': self.pricelist_id,
        }
        so_vals.update(res['value'])
        order_id = self.sale_model.create(cr, uid, so_vals, context=context)

        self.sale_line_model.create(cr, uid, {
            'order_id': order_id,
            'product_id': product_id,
            'name': 'description',
            'type': 'make_to_order',
            'product_uom_qty': qty,
            'price_unit': 500.0,
            'supplier_id': self.supplier_asustek_id,
        }, context=context)

        return self.sale_model.browse(cr, uid, order_id)

    def test_01_service_order_from_sale_order(self):
        """Testeamos creacion de parte de servicio desde sale.order."""
        cr, uid = self.cr, self.uid

        # Creamos y confirmamos una SO
        order = self.create_sale_order(self.partner_id, self.product1_id, 10)
        self.sale_model.action_button_confirm(cr, uid, [order.id])

        self.assertEquals(order.state, 'manual')

        # Lanzamos el wizard para crear partes a partir de la SO
        context = {'active_id': order.id, 'active_ids': [order.id]}
        edate = datetime.now()+relativedelta.relativedelta(months=1, day=1)
        times = 2
        wiz_id = self.create_service_order_model.create(cr, uid, dict(
            type=self.type_installation_id, expected_date=edate,
            times=times, every=1, period='months'),
            context=context)

        service_ids = self.create_service_order_model.create_service_orders(
                            cr, uid, [wiz_id], context)

        self.assertEquals(len(service_ids), times)

        # Browseamos alguna de las service.order
        service = self.service_order_model.browse(cr, uid, service_ids[0])
        so_service = service.sale_order_ids[0].id
        self.assertEquals(so_service, order.id)
